@@ -1,7 +1,7 @@
 /**
  * HTTP/HTTPS client support
  *
- * Copyright (C) 2000-2010 by
+ * Copyright (C) 2000-2013 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -15,9 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.   
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  * 
  */
 #include <setup.h>
@@ -48,6 +48,7 @@ private BOOLEAN __ftp (CONN *C, URL U, CLIENT *c);
 private BOOLEAN __init_connection(CONN *C, URL U, CLIENT *c);
 private void    __increment_failures();
 private int     __select_color(int code);
+private URL     __normalize(URL req, char *location);
 
 
 #ifdef  SIGNAL_CLIENT_PLATFORM
@@ -326,25 +327,25 @@ __http(CONN *C, URL U, CLIENT *client)
       if (my.display)
         DISPLAY(color, "%s%s%s%4d,%s,%d,%6.2f,%7lu,%s,%d,%s",
         time_str, (my.mark)?my.markstr:"", (my.mark)?",":"", client->id, head->head, head->code, 
-        etime, bytes, (my.fullurl)?url_get_absolute(U):url_get_request(U), url_get_ID(U), fmtime
+        etime, bytes, url_get_display(U), url_get_ID(U), fmtime
       );
       else
         DISPLAY(color, "%s%s%s%s,%d,%6.2f,%7lu,%s,%d,%s",
           time_str, (my.mark)?my.markstr:"", (my.mark)?",":"", head->head, head->code, 
-          etime, bytes, (my.fullurl)?url_get_absolute(U):url_get_request(U), url_get_ID(U), fmtime
+          etime, bytes, url_get_display(U), url_get_ID(U), fmtime
         );
     } else {
       if (my.display)
         DISPLAY(
           color, "%s%4d: %s %d %6.2f secs: %7lu bytes ==> %-4s %s", client->id,
           time_str, head->head, head->code, etime, bytes, url_get_method_name(U), 
-          (my.fullurl)?url_get_absolute(U):url_get_request(U)
+		  url_get_display(U)
         ); 
       else
         DISPLAY ( 
           color, "%s%s %d %6.2f secs: %7lu bytes ==> %-4s %s", 
           time_str, head->head, head->code, etime, bytes, url_get_method_name(U), 
-          (my.fullurl)?url_get_absolute(U):url_get_request(U)
+		  url_get_display(U)
         );
     } /* else not my.csv */
     if (my.timestamp) xfree(time_str);
@@ -365,7 +366,13 @@ __http(CONN *C, URL U, CLIENT *client)
     case 302:
     case 307:
       if (my.follow && head->redirect[0]) {
-        redirect_url = new_url(head->redirect);
+        /**  
+         * XXX: What if the server sends us
+         * Location: path/file.htm 
+         *  OR
+         * Location: /path/file.htm
+         */ 
+        redirect_url = __normalize(U, head->redirect); //new_url(head->redirect);
 
         if (empty(url_get_hostname(redirect_url))) { 
           url_set_hostname(redirect_url, url_get_hostname(U));
@@ -750,3 +757,43 @@ __select_color(int code)
   return RED;
 }
 
+private URL
+__normalize(URL req, char *location)
+{
+  URL    ret;
+  char * url;
+  size_t len = strlen(url_get_absolute(req)) + strlen(location) + 32;
+  if (strchr(location, ':') != NULL) {
+    // it's very likely normalized
+    ret = new_url(location);
+    // but we better test it...
+    if (strchr(url_get_hostname(ret), '.') != NULL) {
+      return ret;
+    }
+  }
+  if (strchr(location, '.') != NULL) {
+    // it's *maybe* host/path
+    ret = new_url(location);
+    // so we better test it...
+    if (strchr(url_get_hostname(ret), '.') != NULL) {
+      return ret;
+    }
+    // XXX: do I really need to test for localhost?
+  }
+
+  /**
+   * If we got this far we better construct it...
+   */
+  url = (char*)malloc(len);
+  memset(url, '\0', len);
+
+  if (location[0] == '/') {
+    snprintf(url, len, "%s://%s:%d%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
+  } else {
+    snprintf(url, len, "%s://%s:%d/%s", url_get_scheme_name(req), url_get_hostname(req), url_get_port(req), location);
+  }
+  ret = new_url(url);
+  free(url);
+  return ret;
+}
+
