@@ -15,9 +15,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *--
  */
 #include <setup.h>
 #include <http.h>
@@ -96,12 +97,13 @@ http_get(CONN *C, URL U)
 {
   size_t rlen;
   size_t mlen; 
-  char * protocol; 
-  char * keepalive;
+  char   protocol[16]; 
+  char   keepalive[16];
   char   hoststr[512];
   char   authwww[512];
   char   authpxy[512];
   char   accept[] = "Accept: */*\015\012";
+  char   encoding[512];
   char   *request;
   char   portstr[16];
   char   fullpath[4096];
@@ -119,22 +121,11 @@ http_get(CONN *C, URL U)
   /* Request path based on proxy settings */
   if(auth_get_proxy_required(my.auth)){
     sprintf(
-      fullpath, "%s://%s:%d%s%s%s%s%s", C->encrypt == FALSE?"http":"https", 
-      url_get_hostname(U), url_get_port(U), url_get_request(U), 
-      (url_get_parameters(U)!=NULL && strlen(url_get_parameters(U)) > 1) ? ";" : "",
-      (url_get_parameters(U)!=NULL && strlen(url_get_parameters(U)) > 1) ? url_get_parameters(U) : "",
-      (url_get_query(U)!=NULL && strlen(url_get_query(U)) > 1) ? "?" : "",
-      (url_get_query(U)!=NULL && strlen(url_get_query(U)) > 1) ? url_get_query(U) : ""
+      fullpath, "%s://%s:%d%s", C->encrypt == FALSE?"http":"https", 
+      url_get_hostname(U), url_get_port(U), url_get_request(U)
     );
   } else {
-    sprintf(
-      fullpath, "%s%s%s%s%s", 
-      url_get_request(U), 
-      (url_get_parameters(U)!=NULL && strlen(url_get_parameters(U)) > 1) ? ";" : "",
-      (url_get_parameters(U)!=NULL && strlen(url_get_parameters(U)) > 1) ? url_get_parameters(U) : "",
-      (url_get_query(U)!=NULL && strlen(url_get_query(U)) > 1) ? "?" : "",
-      (url_get_query(U)!=NULL && strlen(url_get_query(U)) > 1) ? url_get_query(U) : ""
-    );
+    sprintf(fullpath, "%s", url_get_request(U));
   }
   if ((url_get_port(U)==80 && C->encrypt==FALSE) || (url_get_port(U)==443 && C->encrypt==FALSE)){
     portstr[0] = '\0';  
@@ -142,9 +133,21 @@ http_get(CONN *C, URL U)
     snprintf(portstr, sizeof portstr, ":%d", url_get_port(U));
   }
 
-  /* HTTP protocol string */
-  protocol  = (my.protocol == TRUE)?"HTTP/1.1":"HTTP/1.0";
-  keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
+  /**
+   * Set the protocol and keepalive strings
+   * based on configuration conditions....
+   */
+  if (my.protocol == FALSE || my.get == TRUE) {
+    snprintf(protocol, sizeof(protocol), "HTTP/1.0");
+  } else {
+    snprintf(protocol, sizeof(protocol), "HTTP/1.1");
+  }
+  if (C->connection.keepalive == TRUE) {
+    snprintf(keepalive, sizeof(keepalive), "keep-alive");
+  } else {
+    snprintf(keepalive, sizeof(keepalive), "close");
+  }
+
   get_cookie_header(pthread_self(), url_get_hostname(U), cookie); 
   if (C->auth.www) {
     if (C->auth.type.www==DIGEST) {
@@ -169,7 +172,7 @@ http_get(CONN *C, URL U)
 
   /* Only send the Host header if one wasn't provided. */
   if (strncasestr(my.extra, "host:", sizeof(my.extra)) == NULL) {
-    rlen = snprintf(hoststr, sizeof(hoststr), "Host: %s%s\015\012", url_get_hostname(U), portstr);
+    rlen = snprintf(hoststr, sizeof(hoststr), "Host: %s\015\012", url_get_hostname(U));
   }
 
   mlen = strlen(url_get_method_name(U)) +
@@ -182,13 +185,17 @@ http_get(CONN *C, URL U)
          strlen((ifmod!=NULL)?ifmod:"") +
          strlen((ifnon!=NULL)?ifnon:"") +
          strlen((strncasecmp(my.extra, "Accept:", 7)==0) ? "" : accept) +
-         strlen((my.get)?"":my.encoding) +
+         sizeof(encoding) +
          strlen(my.uagent) +
          strlen(my.extra) +
          strlen(keepalive) +
          128; 
    request = (char*)xmalloc(mlen);
    memset(request, '\0', mlen);
+   memset(encoding, '\0', sizeof(encoding));
+   if (! my.get) {
+     snprintf(encoding, sizeof(encoding), "Accept-Encoding: %s\015\012", my.encoding); 
+   }
 
   /** 
    * build a request string to pass to the server       
@@ -203,7 +210,7 @@ http_get(CONN *C, URL U)
     "%s"                                   /* ifmod     or empty str */
     "%s"                                   /* ifnon     or empty str */
     "%s"                                   /* Conditional Accept:    */
-    "Accept-Encoding: %s\015\012"          /* my.encoding */
+    "%s"                                   /* encoding */
     "User-Agent: %s\015\012"               /* my uagent   */
     "%s"                                   /* my.extra    */
     "Connection: %s\015\012\015\012",      /* keepalive   */
@@ -214,7 +221,7 @@ http_get(CONN *C, URL U)
     (ifmod!=NULL)?ifmod:"",
     (ifnon!=NULL)?ifnon:"",
     (strncasecmp(my.extra, "Accept:", 7)==0) ? "" : accept,
-    (my.get)?"":my.encoding, my.uagent, my.extra, keepalive 
+    encoding, my.uagent, my.extra, keepalive 
   );
 
   /**
@@ -248,16 +255,19 @@ http_post(CONN *C, URL U)
   char   authwww[128];
   char   authpxy[128]; 
   char   accept[] = "Accept: */*\015\012";
+  char   encoding[512];
   char * request; 
   char   portstr[16];
-  char * protocol; 
-  char * keepalive;
+  char   protocol[16]; 
+  char   keepalive[16];
   char   cookie[MAX_COOKIE_SIZE];
   char   fullpath[4096];
 
-  memset(hoststr, '\0', sizeof(hoststr));
-  memset(cookie,  '\0', sizeof(cookie));
-  memset(portstr, '\0', sizeof portstr);
+  memset(hoststr,  '\0', sizeof(hoststr));
+  memset(cookie,   '\0', sizeof(cookie));
+  memset(portstr,  '\0', sizeof portstr);
+  memset(protocol, '\0', sizeof portstr);
+  memset(keepalive,'\0', sizeof portstr);
 
   if (auth_get_proxy_required(my.auth)) {
    sprintf(
@@ -276,10 +286,20 @@ http_post(CONN *C, URL U)
   }
 
   /** 
-   * HTTP protocol string 
+   * Set the protocol and keepalive strings
+   * based on configuration conditions.... 
    */
-  protocol  = (my.protocol == TRUE)?"HTTP/1.1":"HTTP/1.0";
-  keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
+  if (my.protocol == FALSE || my.get == TRUE) { 
+    snprintf(protocol, sizeof(protocol), "HTTP/1.0");
+  } else {
+    snprintf(protocol, sizeof(protocol), "HTTP/1.1");
+  }
+  if (C->connection.keepalive == TRUE) {
+    snprintf(keepalive, sizeof(keepalive), "keep-alive");
+  } else {
+    snprintf(keepalive, sizeof(keepalive), "close");
+  }
+
   get_cookie_header(pthread_self(), url_get_hostname(U), cookie);
   if (C->auth.www) {
     if (C->auth.type.www==DIGEST) {
@@ -304,7 +324,7 @@ http_post(CONN *C, URL U)
 
   /* Only send the Host header if one wasn't provided. */
   if (strncasestr(my.extra, "host:", sizeof(my.extra)) == NULL) {
-    rlen = snprintf(hoststr, sizeof(hoststr), "Host: %s%s\015\012", url_get_hostname(U), portstr);
+    rlen = snprintf(hoststr, sizeof(hoststr), "Host: %s\015\012", url_get_hostname(U));
   }
 
   mlen = strlen(url_get_method_name(U)) +
@@ -315,14 +335,19 @@ http_post(CONN *C, URL U)
          strlen((C->auth.proxy==TRUE)?authpxy:"") +
          strlen(cookie) +
          strlen((strncasecmp(my.extra, "Accept:", 7)==0) ? "" : accept) +
-         strlen((my.get)?"":my.encoding) +
+         sizeof(encoding) +
          strlen(my.uagent) +
+         strlen(url_get_conttype(U)) +
          strlen(my.extra) +
          strlen(keepalive) + 
          url_get_postlen(U) +
          128; 
    request = (char*)xmalloc(mlen);
    memset(request, '\0', mlen);
+   memset(encoding, '\0', sizeof(encoding));
+   if (! my.get) {
+     snprintf(encoding, sizeof(encoding), "Accept-Encoding: %s\015\012", my.encoding); 
+   }
 
   /** 
    * build a request string to
@@ -336,7 +361,7 @@ http_post(CONN *C, URL U)
     "%s"
     "%s"
     "%s"
-    "Accept-Encoding: %s\015\012"
+    "%s"
     "User-Agent: %s\015\012%s"
     "Connection: %s\015\012"
     "Content-type: %s\015\012"
@@ -346,10 +371,10 @@ http_post(CONN *C, URL U)
     (C->auth.proxy==TRUE)?authpxy:"",
     (strlen(cookie) > 8)?cookie:"", 
     (strncasecmp(my.extra, "Accept:", 7)==0) ? "" : accept,
-    (my.get)?"":my.encoding, my.uagent, my.extra, keepalive, url_get_conttype(U), (long)url_get_postlen(U)
+    encoding, my.uagent, my.extra, keepalive, url_get_conttype(U), (long)url_get_postlen(U)
   ); 
 
-  if (rlen + url_get_postlen(U) < sizeof(request)) {
+  if (rlen < mlen) {
     memcpy(request + rlen, url_get_postdata(U), url_get_postlen(U));
     request[rlen+url_get_postlen(U)] = 0;
   }
@@ -365,6 +390,7 @@ http_post(CONN *C, URL U)
     return FALSE;
   }
 
+  xfree(request);
   return TRUE;
 }
 
@@ -635,6 +661,9 @@ http_read(CONN *C)
       } while (bytes < C->content.length);
     } else {
       do {
+        /*if (my.chunked && C->content.transfer == CHUNKED) {
+          chunk = http_chunk_size(C);
+        } */
         if ((n = socket_read(C, &c, 1)) == 0)
           break;
         echo ("%c", c);
