@@ -1,7 +1,7 @@
 /**
  * Siege environment initialization.
  *
- * Copyright (C) 2000-2010 by
+ * Copyright (C) 2000-2013 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege 
  *
@@ -15,10 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */  
 #include <init.h>
 #include <setup.h>
@@ -64,6 +63,7 @@ init_config( void )
   my.cookies        = TRUE;
   my.csv            = FALSE;
   my.fullurl        = FALSE;
+  my.escape         = TRUE;
   my.secs           = -1;
   my.reps           = MAXREPS; 
   my.bids           = 5;
@@ -92,7 +92,7 @@ init_config( void )
   if((res = pthread_cond_init(&(my.cond), NULL )) != 0)
     NOTIFY(FATAL, "unable to initiate condition");
 
-  if(load_conf(my.rc) < 0){
+  if (load_conf(my.rc) < 0) {
     fprintf( stderr, "****************************************************\n" );
     fprintf( stderr, "siege: could not open %s\n", my.rc );
     fprintf( stderr, "run \'siege.config\' to generate a new .siegerc file\n" );
@@ -100,31 +100,31 @@ init_config( void )
     return -1;
   }
   
-  if(strlen(my.file) < 1){ 
+  if (strlen(my.file) < 1) { 
     snprintf(
       my.file, sizeof( my.file ),
       "%s", URL_FILE
     );
   }
 
-  if(strlen(my.uagent) < 1) 
+  if (strlen(my.uagent) < 1) 
     snprintf( 
       my.uagent, sizeof(my.uagent),
-      "JoeDog/1.00 [en] (X11; I; Siege %s)", version_string 
+      "Mozilla/5.0 (%s) Siege/%s", PLATFORM, version_string 
     );
 
-  if(strlen(my.conttype) < 1) 
+  if (strlen(my.conttype) < 1) 
     snprintf( 
       my.conttype, sizeof(my.conttype),
       "application/x-www-form-urlencoded"
     );
 
-  if(strlen(my.encoding) < 1)
+  if (strlen(my.encoding) < 1)
     snprintf(
       my.encoding, sizeof(my.encoding), "*"
     );
 
-  if(strlen(my.logfile) < 1) 
+  if (strlen(my.logfile) < 1) 
     snprintf( 
       my.logfile, sizeof( my.logfile ),
       "%s", LOG_FILE 
@@ -133,7 +133,7 @@ init_config( void )
 }
 
 int
-show_config( int EXIT )
+show_config(int EXIT)
 {
   char *method;
   switch (my.method) {
@@ -171,6 +171,7 @@ show_config( int EXIT )
   else
     printf( "repetitions:                    n/a\n" );
   printf( "socket timeout:                 %d\n", my.timeout );
+  printf( "accept-encoding:                %s\n", my.encoding);
   printf( "delay:                          %d sec%s\n", my.delay,my.delay>1?"s":"" );
   printf( "internet simulation:            %s\n", my.internet?"true":"false"  );
   printf( "benchmark mode:                 %s\n", my.bench?"true":"false"  );
@@ -194,8 +195,8 @@ show_config( int EXIT )
   else return 0;
 }
 
-static char
-*get_line(FILE *fp)
+static char *
+get_line(FILE *fp)
 {
   char *ptr = NULL;
   char *newline;
@@ -217,8 +218,8 @@ static char
   return ptr;
 } 
 
-static char
-*chomp_line(FILE *fp, char **mystr, int *line_num)
+static char *
+chomp_line(FILE *fp, char **mystr, int *line_num)
 {
   char *ptr;
   while(TRUE){
@@ -253,14 +254,16 @@ load_conf(char *filename)
   H = new_hash();
 
   while ((line = chomp_line(fp, &line, &line_num)) != NULL) {
-    tmp    = line;
+    tmp = trim(line);
     optionptr = option = xstrdup(line);
-    while (*optionptr && !ISSPACE((int)*optionptr) && !ISSEPARATOR(*optionptr))
+    while (*optionptr && !ISSPACE((int)*optionptr) && !ISSEPARATOR(*optionptr)) {
       optionptr++;
+    }
     *optionptr++=0;
-    while(ISSPACE((int)*optionptr) || ISSEPARATOR(*optionptr))
+    while (ISSPACE((int)*optionptr) || ISSEPARATOR(*optionptr)) {
       optionptr++;
-    value  = xstrdup(optionptr);
+    }
+    value = xstrdup(optionptr);
     while (*line)
       line++;  
     while (strstr(option, "$")) {
@@ -490,6 +493,12 @@ load_conf(char *filename)
       else
         my.follow = FALSE;
     }
+    else if (strmatch(option, "url-escaping")) {
+      if (!strncasecmp(value, "true", 4))
+        my.escape = TRUE;
+      else
+        my.escape = FALSE;
+    } 
     else if (strmatch(option, "zero-data-ok")) {
       if (!strncasecmp(value, "true", 4))
         my.zero_ok = TRUE;
