@@ -1,3 +1,25 @@
+/**
+ * URL Support
+ *
+ * Copyright (C) 2013 by
+ * Jeffrey Fulmer - <jeff@joedog.org>, et al.
+ * This file is distributed as part of Siege
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *--
+ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <setup.h>
@@ -7,6 +29,7 @@
 #include <util.h>
 #include <joedog/joedog.h>
 #include <joedog/boolean.h>
+#include <joedog/defs.h>
 
 struct URL_T
 {
@@ -54,6 +77,9 @@ private char *  __url_set_file(URL this, char *str);
 private char *  __url_set_parameters(URL this, char *str);
 private char *  __url_set_query(URL this, char *str);
 private char *  __url_set_fragment(URL this, char *str);
+private char *  __url_escape(const char *s);
+private METHOD  __url_has_method(const char *url);
+
 
 URL
 new_url(char *str)
@@ -70,18 +96,20 @@ new_url(char *str)
 URL
 url_destroy(URL this)
 {
-  xfree(this->username);
-  xfree(this->password);
-  xfree(this->hostname);
-  xfree(this->path);
-  xfree(this->file);
-  xfree(this->query);
-  xfree(this->frag);
-  xfree(this->request);
-  xfree(this->conttype);
-  xfree(this->postdata);
-  xfree(this->etag);
-  xfree(this);
+  if (this!=NULL) {
+    xfree(this->username);
+    xfree(this->password);
+    xfree(this->hostname);
+    xfree(this->path);
+    xfree(this->file);
+    xfree(this->query);
+    xfree(this->frag);
+    xfree(this->request);
+    xfree(this->conttype);
+    xfree(this->postdata);
+    xfree(this->etag);
+    xfree(this);
+  }
   return NULL;  
 }
 
@@ -184,23 +212,35 @@ url_get_scheme(URL this)
   return this->scheme;
 }
 
+public char *
+url_get_display(URL this)
+{
+  if (my.fullurl)
+    return url_get_absolute(this);
+
+  if (this->method == GET)
+    return url_get_request(this);
+
+  return url_get_absolute(this);
+}
+
 public char *
 url_get_scheme_name(URL this)
 {
   switch (this->scheme) {
     case HTTP:
-    return "HTTP";
+    return "http";
     case HTTPS:
-    return "HTTPS";
+    return "https";
     case FTP:
-    return "FTP";
+    return "ftp";
     case PROXY:
-    return "PROXY";
+    return "proxy";
     case UNSUPPORTED:
     default:
-    return "UNSUPPORTED";
+    return "unsupported";
   }
-  return "UNSUPPORTED";
+  return "unsupported";
 }
 
 public char *
@@ -398,11 +438,21 @@ url_dump(URL this)
 private void
 __url_parse(URL this, char *url)
 {
-  char   *ptr;
+  char   *ptr = NULL;
+  char   *esc = NULL;
   char   *post;  
-  size_t len;
-  
-  ptr = __url_set_absolute(this, url);
+
+  /**
+   * URL escaping is in its infancy so we're
+   * going to make it a configurable option. 
+   * see: url-escaping in siegerc.
+   */
+  esc = __url_escape(url);
+  if (my.escape) {
+    ptr = __url_set_absolute(this, esc);
+  } else {
+    ptr = __url_set_absolute(this, url);
+  }
   ptr = __url_set_scheme(this, ptr);
   
   post = strstr(this->url, " POST");
@@ -425,7 +475,7 @@ __url_parse(URL this, char *url)
     this->posttemp   = NULL;
     this->postlen    = 0;
   }
-  
+
   if (__url_has_credentials(ptr)) {
     ptr = __url_set_username(this, ptr);
     ptr = __url_set_password(this, ptr);
@@ -438,16 +488,6 @@ __url_parse(URL this, char *url)
   ptr = __url_set_parameters(this, ptr);
   ptr = __url_set_query(this, ptr);
   ptr = __url_set_fragment(this, ptr);
-
-  /**
-   * we'll combine the path and the file into 
-   * a convenience variable known as 'request'
-   */
-  len = strlen(this->path)+strlen(this->file)+1;
-  this->request = xmalloc(len);
-  memset(this->request, '\0', len);
-  snprintf(this->request, len, "%s%s", this->path, this->file);
-
   return;
 }
 
@@ -467,7 +507,11 @@ __parse_post_data(URL this, char *datap)
   } else {
     this->postdata = xstrdup(datap);
     this->postlen  = strlen(this->postdata);
-    this->conttype = xstrdup("application/x-www-form-urlencoded");
+    if (! empty(my.conttype)) {
+      this->conttype = xstrdup(my.conttype);
+    } else {
+      this->conttype = xstrdup("application/x-www-form-urlencoded");
+    }
     return;
   }
 
@@ -726,7 +770,7 @@ __url_set_path(URL this, char *str)
       this->request[j+1] = '\0'; // no frag
     }
   }
-
+  trim(this->request);
   str += i + 1;
   return str;
 }
@@ -747,6 +791,7 @@ __url_set_file(URL this, char *str)
   this->file = xmalloc(i+1);
   memset(this->file, '\0', i+1);
   memcpy(this->file, str, i);
+  trim(this->file);
 
   /* if there are params or a query string */
   if (str[i] == ';') { 
@@ -843,3 +888,174 @@ __url_set_fragment(URL this, char *str)
   str += i + 1;
   return str;
 }
+
+/**
+ * The following functions provide url encoding. They
+ * were lifted from wget:
+ * Copyright (C) 1995, 1996, 1997, 1998, 2000, 2001
+ * Free Software Foundation, Inc.
+ */
+enum {
+  /* rfc1738 reserved chars, preserved from encoding.  */
+  urlchr_reserved = 1,
+
+  /* rfc1738 unsafe chars, plus some more.  */
+  urlchr_unsafe   = 2
+};
+
+#define urlchr_test(c, mask) (urlchr_table[(unsigned char)(c)] & (mask))
+#define URL_RESERVED_CHAR(c) urlchr_test(c, urlchr_reserved)
+#define URL_UNSAFE_CHAR(c) urlchr_test(c, urlchr_unsafe)
+
+/* Shorthands for the table: */
+#define R  urlchr_reserved
+#define U  urlchr_unsafe
+#define RU R|U
+
+static const unsigned char urlchr_table[256] =
+{
+  U,  U,  U,  U,   U,  U,  U,  U,   /* NUL SOH STX ETX  EOT ENQ ACK BEL */
+  U,  U,  U,  U,   U,  U,  U,  U,   /* BS  HT  LF  VT   FF  CR  SO  SI  */
+  U,  U,  U,  U,   U,  U,  U,  U,   /* DLE DC1 DC2 DC3  DC4 NAK SYN ETB */
+  U,  U,  U,  U,   U,  U,  U,  U,   /* CAN EM  SUB ESC  FS  GS  RS  US  */
+  U,  0,  U, RU,   0,  U,  R,  0,   /* SP  !   "   #    $   %   &   '   */
+  0,  0,  0,  R,   0,  0,  0,  R,   /* (   )   *   +    ,   -   .   /   */
+  0,  0,  0,  0,   0,  0,  0,  0,   /* 0   1   2   3    4   5   6   7   */
+  0,  0, RU,  R,   U,  R,  U,  R,   /* 8   9   :   ;    <   =   >   ?   */
+ RU,  0,  0,  0,   0,  0,  0,  0,   /* @   A   B   C    D   E   F   G   */
+  0,  0,  0,  0,   0,  0,  0,  0,   /* H   I   J   K    L   M   N   O   */
+  0,  0,  0,  0,   0,  0,  0,  0,   /* P   Q   R   S    T   U   V   W   */
+  0,  0,  0, RU,   U, RU,  U,  0,   /* X   Y   Z   [    \   ]   ^   _   */
+  U,  0,  0,  0,   0,  0,  0,  0,   /* `   a   b   c    d   e   f   g   */
+  0,  0,  0,  0,   0,  0,  0,  0,   /* h   i   j   k    l   m   n   o   */
+  0,  0,  0,  0,   0,  0,  0,  0,   /* p   q   r   s    t   u   v   w   */
+  0,  0,  0,  U,   U,  U,  U,  U,   /* x   y   z   {    |   }   ~   DEL */
+
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+  U, U, U, U,  U, U, U, U,  U, U, U, U,  U, U, U, U,
+};
+#undef R
+#undef U
+#undef RU
+
+enum copy_method { CM_DECODE, CM_ENCODE, CM_PASSTHROUGH };
+
+/** 
+ * Decide whether to encode, decode, or pass through the char at P.
+ *  This used to be a macro, but it got a little too convoluted.  
+ */
+static inline enum copy_method
+decide_copy_method (const char *p)
+{
+  if (*p == '%') {
+    if (ISXDIGIT (*(p + 1)) && ISXDIGIT (*(p + 2))) {
+      /**
+       * %xx sequence: decode it, unless it would decode to an
+       * unsafe or a reserved char; in that case, leave it as is.
+       */ 
+      char preempt = X2DIGITS_TO_NUM (*(p + 1), *(p + 2));
+      if (URL_UNSAFE_CHAR (preempt) || URL_RESERVED_CHAR (preempt))
+        return CM_PASSTHROUGH;
+      else
+        return CM_DECODE;
+    } else {
+      return CM_ENCODE;
+    }
+  }
+  else if (URL_UNSAFE_CHAR (*p) && !URL_RESERVED_CHAR (*p))
+    return CM_ENCODE;
+  else
+    return CM_PASSTHROUGH;
+}
+
+static METHOD
+__url_has_method(const char *url)
+{
+   unsigned int i = 0;
+   const char * r = NULL;
+   static const char* const methods[] = {
+     " GET", " HEAD", " POST", " PUT", " TRACE", " DELETE", " OPTIONS", " CONNECT"
+   };
+
+   for (i = 0; i < sizeof(methods) / sizeof(methods[0]); i++) {
+     r = strstr(url, methods[i]);
+     if (r != NULL) return i;
+   }
+
+   return NOMETHOD;
+}
+
+private char *
+__url_escape(const char *s)
+{
+  const char *p1;
+  char *newstr, *p2;
+  int oldlen, newlen;
+
+  int encode_count = 0;
+  int decode_count = 0;
+
+  /** 
+   * FIXME: we're not going to escape siege method
+   * URLS, i.e., things with PUT or POST but if the
+   * path contains spaces they won't be escaped.
+   */
+  if (__url_has_method(s)!=NOMETHOD) {
+    return (char *)s;
+  }  
+
+  /* First, pass through the string to see if there's anything to do,
+     and to calculate the new length.  */
+  for (p1 = s; *p1; p1++) {
+    switch (decide_copy_method (p1)) {
+      case CM_ENCODE:
+        ++encode_count;
+        break;
+      case CM_DECODE:
+        ++decode_count;
+        break;
+      case CM_PASSTHROUGH:
+        break;
+    }
+  }
+
+  if (!encode_count && !decode_count)
+    return (char *)s; /* C const model sucks. */
+
+  oldlen = p1 - s;
+  /* Each encoding adds two characters (hex digits), while each
+     decoding removes two characters.  */
+  newlen = oldlen + 2 * (encode_count - decode_count);
+  newstr = xmalloc (newlen + 1);
+
+  p1 = s;
+  p2 = newstr;
+
+  while (*p1) {
+    switch (decide_copy_method (p1)) {
+      case CM_ENCODE: {
+        unsigned char c = *p1++;
+        *p2++ = '%';
+        *p2++ = XNUM_TO_DIGIT (c >> 4);
+        *p2++ = XNUM_TO_DIGIT (c & 0xf);
+      }
+      break;
+    case CM_DECODE:
+      *p2++ = X2DIGITS_TO_NUM (p1[1], p1[2]);
+      p1 += 3;              /* skip %xx */
+      break;
+    case CM_PASSTHROUGH:
+      *p2++ = *p1++;
+    }
+  }
+  *p2 = '\0';
+  return newstr;
+}
+
