@@ -1,3 +1,25 @@
+/**
+ * URL Support
+ *
+ * Copyright (C) 2013 by
+ * Jeffrey Fulmer - <jeff@joedog.org>, et al.
+ * This file is distributed as part of Siege
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *--
+ */
 #ifndef __URL_H
 #define __URL_H
 #include <stdlib.h>
@@ -19,14 +41,15 @@ extern size_t  URLSIZE;
  * HTTP method 
  */
 typedef enum {
-  HEAD    = 0,
-  GET     = 1,
-  POST    = 2,
-  PUT     = 3,
-  DELETE  = 4,
-  TRACE   = 5,
-  OPTIONS = 6,
-  CONNECT = 7
+  NOMETHOD =  0,
+  HEAD     =  1,
+  GET      =  2,
+  POST     =  3,
+  PUT      =  4,
+  DELETE   =  5,
+  TRACE    =  6,
+  OPTIONS  =  7,
+  CONNECT  =  8,
 } METHOD;
   
 /**
@@ -94,6 +117,13 @@ char *   url_get_query(URL this);
 char *   url_get_fragment(URL this);
 
 
+/* 
+ *  Make a decision about what to display.  Will show absolute url when fullurl
+ *  is set to ture.  Otherwise we check the HTTP method to display the submitted
+ *  URI or the respective line provided from the urls file.
+ */
+char *   url_get_display(URL this);
+
 /**
  * POST method getters
  * <scheme>://<username>:<password>@<hostname>:<port>/<path> POST <params>?<query>#<frag> 
